(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@angular/common'), require('rxjs'), require('rxjs/operators')) :
    typeof define === 'function' && define.amd ? define('ngx-input-search', ['exports', '@angular/core', '@angular/common', 'rxjs', 'rxjs/operators'], factory) :
    (global = global || self, factory(global['ngx-input-search'] = {}, global.ng.core, global.ng.common, global.rxjs, global.rxjs.operators));
}(this, function (exports, core, common, rxjs, operators) { 'use strict';

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    var InputSearchDirective = /** @class */ (function () {
        function InputSearchDirective(el) {
            this.el = el;
            this.debounceTimeDefault = 400;
            this.minStringLengthDefault = 0;
            // Set defaults
            this.debounceTime = this.debounceTimeDefault;
            this.stringLength = this.minStringLengthDefault;
            this.ngxInputSearch = new core.EventEmitter();
            this.stringTooShort = new core.EventEmitter();
            this.emptyInput = new core.EventEmitter();
            this.inputEl = this.el.nativeElement;
        }
        Object.defineProperty(InputSearchDirective.prototype, "debounceTime", {
            get: /**
             * @return {?}
             */
            function () {
                return this._debounceTime;
            },
            /**
             * Indicates how much time in ms we will wait for the users stops typing.
             * By default is 400ms
             */
            set: /**
             * Indicates how much time in ms we will wait for the users stops typing.
             * By default is 400ms
             * @param {?} dbTime
             * @return {?}
             */
            function (dbTime) {
                this._debounceTime = dbTime ? dbTime : this.debounceTimeDefault;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(InputSearchDirective.prototype, "stringLength", {
            get: /**
             * @return {?}
             */
            function () {
                return this._stringLength;
            },
            /**
             * Indicates the minimum length that must have the string to be emitted.
             * By default is 0
             */
            set: /**
             * Indicates the minimum length that must have the string to be emitted.
             * By default is 0
             * @param {?} stLength
             * @return {?}
             */
            function (stLength) {
                this._stringLength =
                    stLength && stLength >= 0 ? stLength : this.minStringLengthDefault;
            },
            enumerable: true,
            configurable: true
        });
        /**
         * @return {?}
         */
        InputSearchDirective.prototype.ngOnInit = /**
         * @return {?}
         */
        function () {
            var _this = this;
            // Subscribe to the input event
            this.inputEventSubs = rxjs.fromEvent(this.inputEl, 'input')
                .pipe(
            // Wait until the user stops writing
            operators.debounceTime(this.debounceTime), 
            // Transform the event to track the input value, to use it in the comparison
            operators.map((/**
             * @param {?} event
             * @return {?}
             */
            function (event) { return ({
                value: ((/** @type {?} */ (event.target))).value.trim(),
                event: event,
            }); })), 
            // Avoid emitting of the input value is the same
            operators.distinctUntilChanged((/**
             * @param {?} prev
             * @param {?} next
             * @return {?}
             */
            function (prev, next) { return prev.value === next.value; })), 
            // Validate the minimum length that must have the string
            operators.filter((/**
             * @param {?} tmpObj
             * @return {?}
             */
            function (tmpObj) {
                if (tmpObj.value.length === 0) {
                    // emit that the input is empty
                    _this.emptyInput.emit();
                    return false;
                }
                if (tmpObj.value.length < _this.stringLength) {
                    // emit that the string length is too short
                    _this.stringTooShort.emit(tmpObj.value);
                    return false;
                }
                else {
                    return true;
                }
            })), 
            // Return only the input event
            operators.map((/**
             * @param {?} tmpObj
             * @return {?}
             */
            function (tmpObj) { return tmpObj.event; })))
                .subscribe((/**
             * @param {?} $event
             * @return {?}
             */
            function ($event) { return _this.ngxInputSearch.next($event); }));
        };
        /**
         * @return {?}
         */
        InputSearchDirective.prototype.ngOnDestroy = /**
         * @return {?}
         */
        function () {
            this.inputEventSubs.unsubscribe();
        };
        InputSearchDirective.decorators = [
            { type: core.Directive, args: [{
                        selector: '[ngxInputSearch]',
                    },] }
        ];
        /** @nocollapse */
        InputSearchDirective.ctorParameters = function () { return [
            { type: core.ElementRef }
        ]; };
        InputSearchDirective.propDecorators = {
            ngxInputSearch: [{ type: core.Output }],
            debounceTime: [{ type: core.Input }],
            stringLength: [{ type: core.Input }],
            stringTooShort: [{ type: core.Output }],
            emptyInput: [{ type: core.Output }]
        };
        return InputSearchDirective;
    }());
    if (false) {
        /**
         * @type {?}
         * @private
         */
        InputSearchDirective.prototype.debounceTimeDefault;
        /**
         * @type {?}
         * @private
         */
        InputSearchDirective.prototype.minStringLengthDefault;
        /**
         * The output to indicate when the user finish to type on the
         * input search
         * @type {?}
         */
        InputSearchDirective.prototype.ngxInputSearch;
        /**
         * @type {?}
         * @private
         */
        InputSearchDirective.prototype._debounceTime;
        /**
         * @type {?}
         * @private
         */
        InputSearchDirective.prototype._stringLength;
        /**
         * Indicates when the string written length is shorter than the minimum
         * defined by the property `stringLength`
         * @type {?}
         */
        InputSearchDirective.prototype.stringTooShort;
        /**
         * Indicates when the input is empty
         * @type {?}
         */
        InputSearchDirective.prototype.emptyInput;
        /**
         * The native element instance of the input
         * @type {?}
         * @private
         */
        InputSearchDirective.prototype.inputEl;
        /**
         * The subscription to the input native event.
         * Useful to unsubscribe when is destroyed to avoid memory leaks
         * @type {?}
         * @private
         */
        InputSearchDirective.prototype.inputEventSubs;
        /**
         * @type {?}
         * @private
         */
        InputSearchDirective.prototype.el;
    }

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    var NgxInputSearchModule = /** @class */ (function () {
        function NgxInputSearchModule() {
        }
        NgxInputSearchModule.decorators = [
            { type: core.NgModule, args: [{
                        imports: [common.CommonModule],
                        declarations: [InputSearchDirective],
                        exports: [InputSearchDirective],
                    },] }
        ];
        return NgxInputSearchModule;
    }());

    exports.NgxInputSearchModule = NgxInputSearchModule;
    exports.ɵa = InputSearchDirective;

    Object.defineProperty(exports, '__esModule', { value: true });

}));
//# sourceMappingURL=ngx-input-search.umd.js.map
