import { EventEmitter, Directive, ElementRef, Output, Input, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { fromEvent } from 'rxjs';
import { debounceTime, map, distinctUntilChanged, filter } from 'rxjs/operators';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var InputSearchDirective = /** @class */ (function () {
    function InputSearchDirective(el) {
        this.el = el;
        this.debounceTimeDefault = 400;
        this.minStringLengthDefault = 0;
        // Set defaults
        this.debounceTime = this.debounceTimeDefault;
        this.stringLength = this.minStringLengthDefault;
        this.ngxInputSearch = new EventEmitter();
        this.stringTooShort = new EventEmitter();
        this.emptyInput = new EventEmitter();
        this.inputEl = this.el.nativeElement;
    }
    Object.defineProperty(InputSearchDirective.prototype, "debounceTime", {
        get: /**
         * @return {?}
         */
        function () {
            return this._debounceTime;
        },
        /**
         * Indicates how much time in ms we will wait for the users stops typing.
         * By default is 400ms
         */
        set: /**
         * Indicates how much time in ms we will wait for the users stops typing.
         * By default is 400ms
         * @param {?} dbTime
         * @return {?}
         */
        function (dbTime) {
            this._debounceTime = dbTime ? dbTime : this.debounceTimeDefault;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(InputSearchDirective.prototype, "stringLength", {
        get: /**
         * @return {?}
         */
        function () {
            return this._stringLength;
        },
        /**
         * Indicates the minimum length that must have the string to be emitted.
         * By default is 0
         */
        set: /**
         * Indicates the minimum length that must have the string to be emitted.
         * By default is 0
         * @param {?} stLength
         * @return {?}
         */
        function (stLength) {
            this._stringLength =
                stLength && stLength >= 0 ? stLength : this.minStringLengthDefault;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    InputSearchDirective.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        var _this = this;
        // Subscribe to the input event
        this.inputEventSubs = fromEvent(this.inputEl, 'input')
            .pipe(
        // Wait until the user stops writing
        debounceTime(this.debounceTime), 
        // Transform the event to track the input value, to use it in the comparison
        map((/**
         * @param {?} event
         * @return {?}
         */
        function (event) { return ({
            value: ((/** @type {?} */ (event.target))).value.trim(),
            event: event,
        }); })), 
        // Avoid emitting of the input value is the same
        distinctUntilChanged((/**
         * @param {?} prev
         * @param {?} next
         * @return {?}
         */
        function (prev, next) { return prev.value === next.value; })), 
        // Validate the minimum length that must have the string
        filter((/**
         * @param {?} tmpObj
         * @return {?}
         */
        function (tmpObj) {
            if (tmpObj.value.length === 0) {
                // emit that the input is empty
                _this.emptyInput.emit();
                return false;
            }
            if (tmpObj.value.length < _this.stringLength) {
                // emit that the string length is too short
                _this.stringTooShort.emit(tmpObj.value);
                return false;
            }
            else {
                return true;
            }
        })), 
        // Return only the input event
        map((/**
         * @param {?} tmpObj
         * @return {?}
         */
        function (tmpObj) { return tmpObj.event; })))
            .subscribe((/**
         * @param {?} $event
         * @return {?}
         */
        function ($event) { return _this.ngxInputSearch.next($event); }));
    };
    /**
     * @return {?}
     */
    InputSearchDirective.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this.inputEventSubs.unsubscribe();
    };
    InputSearchDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[ngxInputSearch]',
                },] }
    ];
    /** @nocollapse */
    InputSearchDirective.ctorParameters = function () { return [
        { type: ElementRef }
    ]; };
    InputSearchDirective.propDecorators = {
        ngxInputSearch: [{ type: Output }],
        debounceTime: [{ type: Input }],
        stringLength: [{ type: Input }],
        stringTooShort: [{ type: Output }],
        emptyInput: [{ type: Output }]
    };
    return InputSearchDirective;
}());
if (false) {
    /**
     * @type {?}
     * @private
     */
    InputSearchDirective.prototype.debounceTimeDefault;
    /**
     * @type {?}
     * @private
     */
    InputSearchDirective.prototype.minStringLengthDefault;
    /**
     * The output to indicate when the user finish to type on the
     * input search
     * @type {?}
     */
    InputSearchDirective.prototype.ngxInputSearch;
    /**
     * @type {?}
     * @private
     */
    InputSearchDirective.prototype._debounceTime;
    /**
     * @type {?}
     * @private
     */
    InputSearchDirective.prototype._stringLength;
    /**
     * Indicates when the string written length is shorter than the minimum
     * defined by the property `stringLength`
     * @type {?}
     */
    InputSearchDirective.prototype.stringTooShort;
    /**
     * Indicates when the input is empty
     * @type {?}
     */
    InputSearchDirective.prototype.emptyInput;
    /**
     * The native element instance of the input
     * @type {?}
     * @private
     */
    InputSearchDirective.prototype.inputEl;
    /**
     * The subscription to the input native event.
     * Useful to unsubscribe when is destroyed to avoid memory leaks
     * @type {?}
     * @private
     */
    InputSearchDirective.prototype.inputEventSubs;
    /**
     * @type {?}
     * @private
     */
    InputSearchDirective.prototype.el;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var NgxInputSearchModule = /** @class */ (function () {
    function NgxInputSearchModule() {
    }
    NgxInputSearchModule.decorators = [
        { type: NgModule, args: [{
                    imports: [CommonModule],
                    declarations: [InputSearchDirective],
                    exports: [InputSearchDirective],
                },] }
    ];
    return NgxInputSearchModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { NgxInputSearchModule, InputSearchDirective as ɵa };
//# sourceMappingURL=ngx-input-search.js.map
